
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#ifndef QFU_ENUM_TYPES_H
#define QFU_ENUM_TYPES_H
#include "qfu-image.h"
#include "qfu-qdl-message.h"
#include "qfu-dload-message.h"
#include "qfu-sahara-message.h"

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "../git/src/qmi-firmware-update/qfu-dload-message.h" */
GType qfu_dload_cmd_get_type (void) G_GNUC_CONST;
#define QFU_TYPE_DLOAD_CMD (qfu_dload_cmd_get_type ())

/* Define type-specific symbols */
#define __QFU_DLOAD_CMD_IS_ENUM__

const gchar *qfu_dload_cmd_get_string (QfuDloadCmd val);

/* enumerations from "../git/src/qmi-firmware-update/qfu-image.h" */
GType qfu_image_type_get_type (void) G_GNUC_CONST;
#define QFU_TYPE_IMAGE_TYPE (qfu_image_type_get_type ())

/* Define type-specific symbols */
#define __QFU_IMAGE_TYPE_IS_ENUM__

const gchar *qfu_image_type_get_string (QfuImageType val);

/* enumerations from "../git/src/qmi-firmware-update/qfu-qdl-message.h" */
GType qfu_qdl_cmd_get_type (void) G_GNUC_CONST;
#define QFU_TYPE_QDL_CMD (qfu_qdl_cmd_get_type ())

/* Define type-specific symbols */
#define __QFU_QDL_CMD_IS_ENUM__

const gchar *qfu_qdl_cmd_get_string (QfuQdlCmd val);

/* enumerations from "../git/src/qmi-firmware-update/qfu-sahara-message.h" */
GType qfu_sahara_cmd_get_type (void) G_GNUC_CONST;
#define QFU_TYPE_SAHARA_CMD (qfu_sahara_cmd_get_type ())

/* Define type-specific symbols */
#define __QFU_SAHARA_CMD_IS_ENUM__

const gchar *qfu_sahara_cmd_get_string (QfuSaharaCmd val);
GType qfu_sahara_mode_get_type (void) G_GNUC_CONST;
#define QFU_TYPE_SAHARA_MODE (qfu_sahara_mode_get_type ())

/* Define type-specific symbols */
#define __QFU_SAHARA_MODE_IS_ENUM__

const gchar *qfu_sahara_mode_get_string (QfuSaharaMode val);
GType qfu_sahara_status_get_type (void) G_GNUC_CONST;
#define QFU_TYPE_SAHARA_STATUS (qfu_sahara_status_get_type ())

/* Define type-specific symbols */
#define __QFU_SAHARA_STATUS_IS_ENUM__

const gchar *qfu_sahara_status_get_string (QfuSaharaStatus val);
G_END_DECLS

#endif /* __QFUENUM_TYPES_H__ */

/* Generated data ends here */

