
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#ifndef __LIBQMI_GLIB_ENUM_TYPES_PRIVATE_H__
#define __LIBQMI_GLIB_ENUM_TYPES_PRIVATE_H__
#include "qmi-enums-private.h"

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "../git/src/libqmi-glib/qmi-enums-private.h" */
GType qmi_ctl_data_format_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_CTL_DATA_FORMAT (qmi_ctl_data_format_get_type ())

/* Define type-specific symbols */
#define __QMI_CTL_DATA_FORMAT_IS_ENUM__

const gchar *qmi_ctl_data_format_get_string (QmiCtlDataFormat val);
GType qmi_ctl_data_link_protocol_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_CTL_DATA_LINK_PROTOCOL (qmi_ctl_data_link_protocol_get_type ())

/* Define type-specific symbols */
#define __QMI_CTL_DATA_LINK_PROTOCOL_IS_ENUM__

const gchar *qmi_ctl_data_link_protocol_get_string (QmiCtlDataLinkProtocol val);
G_END_DECLS

#endif /* __LIBQMI_GLIB_ENUM_TYPES_PRIVATE_H__ */

/* Generated data ends here */

