
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#ifndef __LIBQMI_GLIB_FLAG_TYPES_H__
#define __LIBQMI_GLIB_FLAG_TYPES_H__
#include "qmi-enums.h"
#include "qmi-enums-wds.h"
#include "qmi-enums-dms.h"
#include "qmi-enums-nas.h"
#include "qmi-enums-wms.h"
#include "qmi-enums-pds.h"
#include "qmi-enums-pdc.h"
#include "qmi-enums-pbm.h"
#include "qmi-enums-uim.h"
#include "qmi-enums-sar.h"
#include "qmi-enums-oma.h"
#include "qmi-enums-wda.h"
#include "qmi-enums-voice.h"
#include "qmi-enums-loc.h"
#include "qmi-enums-qos.h"
#include "qmi-enums-gas.h"
#include "qmi-enums-dsd.h"
#include "qmi-enums-fox.h"
#include "qmi-enums-ssc.h"
#include "qmi-device.h"

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "../git/src/libqmi-glib/qmi-device.h" */
GType qmi_device_open_flags_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_DEVICE_OPEN_FLAGS (qmi_device_open_flags_get_type ())

/* Define type-specific symbols */
#define __QMI_DEVICE_OPEN_FLAGS_IS_FLAGS__

/**
 * qmi_device_open_flags_build_string_from_mask:
 * @mask: bitmask of QmiDeviceOpenFlags values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiDeviceOpenFlags in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_device_open_flags_build_string_from_mask (QmiDeviceOpenFlags mask);
GType qmi_device_release_client_flags_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_DEVICE_RELEASE_CLIENT_FLAGS (qmi_device_release_client_flags_get_type ())

/* Define type-specific symbols */
#define __QMI_DEVICE_RELEASE_CLIENT_FLAGS_IS_FLAGS__

/**
 * qmi_device_release_client_flags_build_string_from_mask:
 * @mask: bitmask of QmiDeviceReleaseClientFlags values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiDeviceReleaseClientFlags in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_device_release_client_flags_build_string_from_mask (QmiDeviceReleaseClientFlags mask);
GType qmi_device_add_link_flags_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_DEVICE_ADD_LINK_FLAGS (qmi_device_add_link_flags_get_type ())

/* Define type-specific symbols */
#define __QMI_DEVICE_ADD_LINK_FLAGS_IS_FLAGS__

/**
 * qmi_device_add_link_flags_build_string_from_mask:
 * @mask: bitmask of QmiDeviceAddLinkFlags values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiDeviceAddLinkFlags in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.30
 */
gchar *qmi_device_add_link_flags_build_string_from_mask (QmiDeviceAddLinkFlags mask);

/* enumerations from "../git/src/libqmi-glib/qmi-enums-dms.h" */
GType qmi_dms_power_state_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_DMS_POWER_STATE (qmi_dms_power_state_get_type ())

/* Define type-specific symbols */
#define __QMI_DMS_POWER_STATE_IS_FLAGS__

/**
 * qmi_dms_power_state_build_string_from_mask:
 * @mask: bitmask of QmiDmsPowerState values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiDmsPowerState in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_dms_power_state_build_string_from_mask (QmiDmsPowerState mask);
GType qmi_dms_offline_reason_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_DMS_OFFLINE_REASON (qmi_dms_offline_reason_get_type ())

/* Define type-specific symbols */
#define __QMI_DMS_OFFLINE_REASON_IS_FLAGS__

/**
 * qmi_dms_offline_reason_build_string_from_mask:
 * @mask: bitmask of QmiDmsOfflineReason values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiDmsOfflineReason in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_dms_offline_reason_build_string_from_mask (QmiDmsOfflineReason mask);

/* enumerations from "../git/src/libqmi-glib/qmi-enums-loc.h" */
GType qmi_loc_technology_used_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_LOC_TECHNOLOGY_USED (qmi_loc_technology_used_get_type ())

/* Define type-specific symbols */
#define __QMI_LOC_TECHNOLOGY_USED_IS_FLAGS__

/**
 * qmi_loc_technology_used_build_string_from_mask:
 * @mask: bitmask of QmiLocTechnologyUsed values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiLocTechnologyUsed in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.22
 */
gchar *qmi_loc_technology_used_build_string_from_mask (QmiLocTechnologyUsed mask);
GType qmi_loc_satellite_valid_information_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_LOC_SATELLITE_VALID_INFORMATION (qmi_loc_satellite_valid_information_get_type ())

/* Define type-specific symbols */
#define __QMI_LOC_SATELLITE_VALID_INFORMATION_IS_FLAGS__

/**
 * qmi_loc_satellite_valid_information_build_string_from_mask:
 * @mask: bitmask of QmiLocSatelliteValidInformation values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiLocSatelliteValidInformation in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.22
 */
gchar *qmi_loc_satellite_valid_information_build_string_from_mask (QmiLocSatelliteValidInformation mask);
GType qmi_loc_server_address_type_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_LOC_SERVER_ADDRESS_TYPE (qmi_loc_server_address_type_get_type ())

/* Define type-specific symbols */
#define __QMI_LOC_SERVER_ADDRESS_TYPE_IS_FLAGS__

/**
 * qmi_loc_server_address_type_build_string_from_mask:
 * @mask: bitmask of QmiLocServerAddressType values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiLocServerAddressType in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.22
 */
gchar *qmi_loc_server_address_type_build_string_from_mask (QmiLocServerAddressType mask);
GType qmi_loc_delete_sv_info_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_LOC_DELETE_SV_INFO (qmi_loc_delete_sv_info_get_type ())

/* Define type-specific symbols */
#define __QMI_LOC_DELETE_SV_INFO_IS_FLAGS__

/**
 * qmi_loc_delete_sv_info_build_string_from_mask:
 * @mask: bitmask of QmiLocDeleteSvInfo values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiLocDeleteSvInfo in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.22
 */
gchar *qmi_loc_delete_sv_info_build_string_from_mask (QmiLocDeleteSvInfo mask);
GType qmi_loc_delete_cell_database_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_LOC_DELETE_CELL_DATABASE (qmi_loc_delete_cell_database_get_type ())

/* Define type-specific symbols */
#define __QMI_LOC_DELETE_CELL_DATABASE_IS_FLAGS__

/**
 * qmi_loc_delete_cell_database_build_string_from_mask:
 * @mask: bitmask of QmiLocDeleteCellDatabase values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiLocDeleteCellDatabase in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.22
 */
gchar *qmi_loc_delete_cell_database_build_string_from_mask (QmiLocDeleteCellDatabase mask);
GType qmi_loc_delete_clock_info_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_LOC_DELETE_CLOCK_INFO (qmi_loc_delete_clock_info_get_type ())

/* Define type-specific symbols */
#define __QMI_LOC_DELETE_CLOCK_INFO_IS_FLAGS__

/**
 * qmi_loc_delete_clock_info_build_string_from_mask:
 * @mask: bitmask of QmiLocDeleteClockInfo values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiLocDeleteClockInfo in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.22
 */
gchar *qmi_loc_delete_clock_info_build_string_from_mask (QmiLocDeleteClockInfo mask);
GType qmi_loc_nmea_type_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_LOC_NMEA_TYPE (qmi_loc_nmea_type_get_type ())

/* Define type-specific symbols */
#define __QMI_LOC_NMEA_TYPE_IS_FLAGS__

/**
 * qmi_loc_nmea_type_build_string_from_mask:
 * @mask: bitmask of QmiLocNmeaType values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiLocNmeaType in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.26
 */
gchar *qmi_loc_nmea_type_build_string_from_mask (QmiLocNmeaType mask);

/* enumerations from "../git/src/libqmi-glib/qmi-enums-nas.h" */
GType qmi_nas_signal_strength_request_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_NAS_SIGNAL_STRENGTH_REQUEST (qmi_nas_signal_strength_request_get_type ())

/* Define type-specific symbols */
#define __QMI_NAS_SIGNAL_STRENGTH_REQUEST_IS_FLAGS__

/**
 * qmi_nas_signal_strength_request_build_string_from_mask:
 * @mask: bitmask of QmiNasSignalStrengthRequest values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiNasSignalStrengthRequest in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_nas_signal_strength_request_build_string_from_mask (QmiNasSignalStrengthRequest mask);
GType qmi_nas_network_scan_type_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_NAS_NETWORK_SCAN_TYPE (qmi_nas_network_scan_type_get_type ())

/* Define type-specific symbols */
#define __QMI_NAS_NETWORK_SCAN_TYPE_IS_FLAGS__

/**
 * qmi_nas_network_scan_type_build_string_from_mask:
 * @mask: bitmask of QmiNasNetworkScanType values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiNasNetworkScanType in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_nas_network_scan_type_build_string_from_mask (QmiNasNetworkScanType mask);
GType qmi_nas_network_status_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_NAS_NETWORK_STATUS (qmi_nas_network_status_get_type ())

/* Define type-specific symbols */
#define __QMI_NAS_NETWORK_STATUS_IS_FLAGS__

/**
 * qmi_nas_network_status_build_string_from_mask:
 * @mask: bitmask of QmiNasNetworkStatus values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiNasNetworkStatus in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_nas_network_status_build_string_from_mask (QmiNasNetworkStatus mask);
GType qmi_nas_plmn_access_technology_identifier_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_NAS_PLMN_ACCESS_TECHNOLOGY_IDENTIFIER (qmi_nas_plmn_access_technology_identifier_get_type ())

/* Define type-specific symbols */
#define __QMI_NAS_PLMN_ACCESS_TECHNOLOGY_IDENTIFIER_IS_FLAGS__

/**
 * qmi_nas_plmn_access_technology_identifier_build_string_from_mask:
 * @mask: bitmask of QmiNasPlmnAccessTechnologyIdentifier values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiNasPlmnAccessTechnologyIdentifier in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.30
 */
gchar *qmi_nas_plmn_access_technology_identifier_build_string_from_mask (QmiNasPlmnAccessTechnologyIdentifier mask);
GType qmi_nas_radio_technology_preference_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_NAS_RADIO_TECHNOLOGY_PREFERENCE (qmi_nas_radio_technology_preference_get_type ())

/* Define type-specific symbols */
#define __QMI_NAS_RADIO_TECHNOLOGY_PREFERENCE_IS_FLAGS__

/**
 * qmi_nas_radio_technology_preference_build_string_from_mask:
 * @mask: bitmask of QmiNasRadioTechnologyPreference values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiNasRadioTechnologyPreference in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_nas_radio_technology_preference_build_string_from_mask (QmiNasRadioTechnologyPreference mask);
GType qmi_nas_rat_mode_preference_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_NAS_RAT_MODE_PREFERENCE (qmi_nas_rat_mode_preference_get_type ())

/* Define type-specific symbols */
#define __QMI_NAS_RAT_MODE_PREFERENCE_IS_FLAGS__

/**
 * qmi_nas_rat_mode_preference_build_string_from_mask:
 * @mask: bitmask of QmiNasRatModePreference values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiNasRatModePreference in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_nas_rat_mode_preference_build_string_from_mask (QmiNasRatModePreference mask);
GType qmi_nas_td_scdma_band_preference_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_NAS_TD_SCDMA_BAND_PREFERENCE (qmi_nas_td_scdma_band_preference_get_type ())

/* Define type-specific symbols */
#define __QMI_NAS_TD_SCDMA_BAND_PREFERENCE_IS_FLAGS__

/**
 * qmi_nas_td_scdma_band_preference_build_string_from_mask:
 * @mask: bitmask of QmiNasTdScdmaBandPreference values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiNasTdScdmaBandPreference in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_nas_td_scdma_band_preference_build_string_from_mask (QmiNasTdScdmaBandPreference mask);
GType qmi_nas_network_name_display_condition_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_NAS_NETWORK_NAME_DISPLAY_CONDITION (qmi_nas_network_name_display_condition_get_type ())

/* Define type-specific symbols */
#define __QMI_NAS_NETWORK_NAME_DISPLAY_CONDITION_IS_FLAGS__

/**
 * qmi_nas_network_name_display_condition_build_string_from_mask:
 * @mask: bitmask of QmiNasNetworkNameDisplayCondition values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiNasNetworkNameDisplayCondition in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.18
 */
gchar *qmi_nas_network_name_display_condition_build_string_from_mask (QmiNasNetworkNameDisplayCondition mask);

/* enumerations from "../git/src/libqmi-glib/qmi-enums-pbm.h" */
GType qmi_pbm_event_registration_flag_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_PBM_EVENT_REGISTRATION_FLAG (qmi_pbm_event_registration_flag_get_type ())

/* Define type-specific symbols */
#define __QMI_PBM_EVENT_REGISTRATION_FLAG_IS_FLAGS__

/**
 * qmi_pbm_event_registration_flag_build_string_from_mask:
 * @mask: bitmask of QmiPbmEventRegistrationFlag values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiPbmEventRegistrationFlag in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.6
 */
gchar *qmi_pbm_event_registration_flag_build_string_from_mask (QmiPbmEventRegistrationFlag mask);
GType qmi_pbm_phonebook_type_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_PBM_PHONEBOOK_TYPE (qmi_pbm_phonebook_type_get_type ())

/* Define type-specific symbols */
#define __QMI_PBM_PHONEBOOK_TYPE_IS_FLAGS__

/**
 * qmi_pbm_phonebook_type_build_string_from_mask:
 * @mask: bitmask of QmiPbmPhonebookType values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiPbmPhonebookType in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.6
 */
gchar *qmi_pbm_phonebook_type_build_string_from_mask (QmiPbmPhonebookType mask);

/* enumerations from "../git/src/libqmi-glib/qmi-enums-pds.h" */
GType qmi_pds_data_valid_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_PDS_DATA_VALID (qmi_pds_data_valid_get_type ())

/* Define type-specific symbols */
#define __QMI_PDS_DATA_VALID_IS_FLAGS__

/**
 * qmi_pds_data_valid_build_string_from_mask:
 * @mask: bitmask of QmiPdsDataValid values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiPdsDataValid in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_pds_data_valid_build_string_from_mask (QmiPdsDataValid mask);

/* enumerations from "../git/src/libqmi-glib/qmi-enums-uim.h" */
GType qmi_uim_event_registration_flag_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_UIM_EVENT_REGISTRATION_FLAG (qmi_uim_event_registration_flag_get_type ())

/* Define type-specific symbols */
#define __QMI_UIM_EVENT_REGISTRATION_FLAG_IS_FLAGS__

/**
 * qmi_uim_event_registration_flag_build_string_from_mask:
 * @mask: bitmask of QmiUimEventRegistrationFlag values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiUimEventRegistrationFlag in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.22.4
 */
gchar *qmi_uim_event_registration_flag_build_string_from_mask (QmiUimEventRegistrationFlag mask);
GType qmi_uim_security_attribute_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_UIM_SECURITY_ATTRIBUTE (qmi_uim_security_attribute_get_type ())

/* Define type-specific symbols */
#define __QMI_UIM_SECURITY_ATTRIBUTE_IS_FLAGS__

/**
 * qmi_uim_security_attribute_build_string_from_mask:
 * @mask: bitmask of QmiUimSecurityAttribute values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiUimSecurityAttribute in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.6
 */
gchar *qmi_uim_security_attribute_build_string_from_mask (QmiUimSecurityAttribute mask);
GType qmi_uim_configuration_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_UIM_CONFIGURATION (qmi_uim_configuration_get_type ())

/* Define type-specific symbols */
#define __QMI_UIM_CONFIGURATION_IS_FLAGS__

/**
 * qmi_uim_configuration_build_string_from_mask:
 * @mask: bitmask of QmiUimConfiguration values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiUimConfiguration in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.30
 */
gchar *qmi_uim_configuration_build_string_from_mask (QmiUimConfiguration mask);

/* enumerations from "../git/src/libqmi-glib/qmi-enums-voice.h" */
GType qmi_voice_wcdma_amr_status_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_VOICE_WCDMA_AMR_STATUS (qmi_voice_wcdma_amr_status_get_type ())

/* Define type-specific symbols */
#define __QMI_VOICE_WCDMA_AMR_STATUS_IS_FLAGS__

/**
 * qmi_voice_wcdma_amr_status_build_string_from_mask:
 * @mask: bitmask of QmiVoiceWcdmaAmrStatus values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiVoiceWcdmaAmrStatus in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.14
 */
gchar *qmi_voice_wcdma_amr_status_build_string_from_mask (QmiVoiceWcdmaAmrStatus mask);

/* enumerations from "../git/src/libqmi-glib/qmi-enums-wds.h" */
GType qmi_wds_technology_preference_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_TECHNOLOGY_PREFERENCE (qmi_wds_technology_preference_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_TECHNOLOGY_PREFERENCE_IS_FLAGS__

/**
 * qmi_wds_technology_preference_build_string_from_mask:
 * @mask: bitmask of QmiWdsTechnologyPreference values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsTechnologyPreference in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_wds_technology_preference_build_string_from_mask (QmiWdsTechnologyPreference mask);
GType qmi_wds_rat_3gpp2_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_RAT_3GPP2 (qmi_wds_rat_3gpp2_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_RAT_3GPP2_IS_FLAGS__

/**
 * qmi_wds_rat_3gpp2_build_string_from_mask:
 * @mask: bitmask of QmiWdsRat3gpp2 values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsRat3gpp2 in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_wds_rat_3gpp2_build_string_from_mask (QmiWdsRat3gpp2 mask);
GType qmi_wds_rat_3gpp_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_RAT_3GPP (qmi_wds_rat_3gpp_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_RAT_3GPP_IS_FLAGS__

/**
 * qmi_wds_rat_3gpp_build_string_from_mask:
 * @mask: bitmask of QmiWdsRat3gpp values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsRat3gpp in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_wds_rat_3gpp_build_string_from_mask (QmiWdsRat3gpp mask);
GType qmi_wds_so_cdma1x_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_SO_CDMA1X (qmi_wds_so_cdma1x_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_SO_CDMA1X_IS_FLAGS__

/**
 * qmi_wds_so_cdma1x_build_string_from_mask:
 * @mask: bitmask of QmiWdsSoCdma1x values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsSoCdma1x in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_wds_so_cdma1x_build_string_from_mask (QmiWdsSoCdma1x mask);
GType qmi_wds_so_evdo_rev0_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_SO_EVDO_REV0 (qmi_wds_so_evdo_rev0_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_SO_EVDO_REV0_IS_FLAGS__

/**
 * qmi_wds_so_evdo_rev0_build_string_from_mask:
 * @mask: bitmask of QmiWdsSoEvdoRev0 values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsSoEvdoRev0 in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.18
 */
gchar *qmi_wds_so_evdo_rev0_build_string_from_mask (QmiWdsSoEvdoRev0 mask);
GType qmi_wds_so_evdo_reva_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_SO_EVDO_REVA (qmi_wds_so_evdo_reva_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_SO_EVDO_REVA_IS_FLAGS__

/**
 * qmi_wds_so_evdo_reva_build_string_from_mask:
 * @mask: bitmask of QmiWdsSoEvdoRevA values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsSoEvdoRevA in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_wds_so_evdo_reva_build_string_from_mask (QmiWdsSoEvdoRevA mask);
GType qmi_wds_so_evdo_revb_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_SO_EVDO_REVB (qmi_wds_so_evdo_revb_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_SO_EVDO_REVB_IS_FLAGS__

/**
 * qmi_wds_so_evdo_revb_build_string_from_mask:
 * @mask: bitmask of QmiWdsSoEvdoRevB values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsSoEvdoRevB in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.18
 */
gchar *qmi_wds_so_evdo_revb_build_string_from_mask (QmiWdsSoEvdoRevB mask);
GType qmi_wds_requested_settings_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_REQUESTED_SETTINGS (qmi_wds_requested_settings_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_REQUESTED_SETTINGS_IS_FLAGS__

/**
 * qmi_wds_requested_settings_build_string_from_mask:
 * @mask: bitmask of QmiWdsRequestedSettings values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsRequestedSettings in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.32
 */
gchar *qmi_wds_requested_settings_build_string_from_mask (QmiWdsRequestedSettings mask);
GType qmi_wds_authentication_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_AUTHENTICATION (qmi_wds_authentication_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_AUTHENTICATION_IS_FLAGS__

/**
 * qmi_wds_authentication_build_string_from_mask:
 * @mask: bitmask of QmiWdsAuthentication values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsAuthentication in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.0
 */
gchar *qmi_wds_authentication_build_string_from_mask (QmiWdsAuthentication mask);
GType qmi_wds_packet_statistics_mask_flag_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_PACKET_STATISTICS_MASK_FLAG (qmi_wds_packet_statistics_mask_flag_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_PACKET_STATISTICS_MASK_FLAG_IS_FLAGS__

/**
 * qmi_wds_packet_statistics_mask_flag_build_string_from_mask:
 * @mask: bitmask of QmiWdsPacketStatisticsMaskFlag values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsPacketStatisticsMaskFlag in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.6
 */
gchar *qmi_wds_packet_statistics_mask_flag_build_string_from_mask (QmiWdsPacketStatisticsMaskFlag mask);
GType qmi_wds_set_event_report_transfer_statistics_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_SET_EVENT_REPORT_TRANSFER_STATISTICS (qmi_wds_set_event_report_transfer_statistics_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_SET_EVENT_REPORT_TRANSFER_STATISTICS_IS_FLAGS__

/**
 * qmi_wds_set_event_report_transfer_statistics_build_string_from_mask:
 * @mask: bitmask of QmiWdsSetEventReportTransferStatistics values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsSetEventReportTransferStatistics in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.18
 */
gchar *qmi_wds_set_event_report_transfer_statistics_build_string_from_mask (QmiWdsSetEventReportTransferStatistics mask);
GType qmi_wds_extended_data_bearer_technology_3gpp_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_EXTENDED_DATA_BEARER_TECHNOLOGY_3GPP (qmi_wds_extended_data_bearer_technology_3gpp_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_EXTENDED_DATA_BEARER_TECHNOLOGY_3GPP_IS_FLAGS__

/**
 * qmi_wds_extended_data_bearer_technology_3gpp_build_string_from_mask:
 * @mask: bitmask of QmiWdsExtendedDataBearerTechnology3gpp values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsExtendedDataBearerTechnology3gpp in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.18
 */
gchar *qmi_wds_extended_data_bearer_technology_3gpp_build_string_from_mask (QmiWdsExtendedDataBearerTechnology3gpp mask);
GType qmi_wds_extended_data_bearer_technology_3gpp2_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_WDS_EXTENDED_DATA_BEARER_TECHNOLOGY_3GPP2 (qmi_wds_extended_data_bearer_technology_3gpp2_get_type ())

/* Define type-specific symbols */
#define __QMI_WDS_EXTENDED_DATA_BEARER_TECHNOLOGY_3GPP2_IS_FLAGS__

/**
 * qmi_wds_extended_data_bearer_technology_3gpp2_build_string_from_mask:
 * @mask: bitmask of QmiWdsExtendedDataBearerTechnology3gpp2 values.
 *
 * Builds a string containing a comma-separated list of nicknames for
 * each #QmiWdsExtendedDataBearerTechnology3gpp2 in @mask.
 *
 * Returns: (transfer full): a string with the list of nicknames, or %NULL if none given. The returned value should be freed with g_free().
 * Since: 1.18
 */
gchar *qmi_wds_extended_data_bearer_technology_3gpp2_build_string_from_mask (QmiWdsExtendedDataBearerTechnology3gpp2 mask);
G_END_DECLS

#endif /* __LIBQMI_GLIB_FLAG_TYPES_H__ */

/* Generated data ends here */

