
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "qmi-enum-types-private.h"
/* enumerations from "../git/src/libqmi-glib/qmi-enums-private.h" */
static const GEnumValue qmi_ctl_data_format_values[] = {
    { QMI_CTL_DATA_FORMAT_QOS_FLOW_HEADER_ABSENT, "QMI_CTL_DATA_FORMAT_QOS_FLOW_HEADER_ABSENT", "absent" },
    { QMI_CTL_DATA_FORMAT_QOS_FLOW_HEADER_PRESENT, "QMI_CTL_DATA_FORMAT_QOS_FLOW_HEADER_PRESENT", "present" },
    { 0, NULL, NULL }
};

/* Define type-specific symbols */

GType
qmi_ctl_data_format_get_type (void)
{
    static gsize g_define_type_id_initialized = 0;

    if (g_once_init_enter (&g_define_type_id_initialized)) {
        GType g_define_type_id =
            g_enum_register_static (g_intern_static_string ("QmiCtlDataFormat"),
                                      qmi_ctl_data_format_values);
        g_once_init_leave (&g_define_type_id_initialized, g_define_type_id);
    }

    return g_define_type_id_initialized;
}

/* Enum-specific method to get the value as a string.
 * We get the nick of the GEnumValue. Note that this will be
 * valid even if the GEnumClass is not referenced anywhere. */
const gchar *
qmi_ctl_data_format_get_string (QmiCtlDataFormat val)
{
    guint i;

    for (i = 0; qmi_ctl_data_format_values[i].value_nick; i++) {
      if ((gint)val == qmi_ctl_data_format_values[i].value)
            return qmi_ctl_data_format_values[i].value_nick;
    }

    return NULL;
}
static const GEnumValue qmi_ctl_data_link_protocol_values[] = {
    { QMI_CTL_DATA_LINK_PROTOCOL_UNKNOWN, "QMI_CTL_DATA_LINK_PROTOCOL_UNKNOWN", "unknown" },
    { QMI_CTL_DATA_LINK_PROTOCOL_802_3, "QMI_CTL_DATA_LINK_PROTOCOL_802_3", "802-3" },
    { QMI_CTL_DATA_LINK_PROTOCOL_RAW_IP, "QMI_CTL_DATA_LINK_PROTOCOL_RAW_IP", "raw-ip" },
    { 0, NULL, NULL }
};

/* Define type-specific symbols */

GType
qmi_ctl_data_link_protocol_get_type (void)
{
    static gsize g_define_type_id_initialized = 0;

    if (g_once_init_enter (&g_define_type_id_initialized)) {
        GType g_define_type_id =
            g_enum_register_static (g_intern_static_string ("QmiCtlDataLinkProtocol"),
                                      qmi_ctl_data_link_protocol_values);
        g_once_init_leave (&g_define_type_id_initialized, g_define_type_id);
    }

    return g_define_type_id_initialized;
}

/* Enum-specific method to get the value as a string.
 * We get the nick of the GEnumValue. Note that this will be
 * valid even if the GEnumClass is not referenced anywhere. */
const gchar *
qmi_ctl_data_link_protocol_get_string (QmiCtlDataLinkProtocol val)
{
    guint i;

    for (i = 0; qmi_ctl_data_link_protocol_values[i].value_nick; i++) {
      if ((gint)val == qmi_ctl_data_link_protocol_values[i].value)
            return qmi_ctl_data_link_protocol_values[i].value_nick;
    }

    return NULL;
}

/* Generated data ends here */

